@@ -41,7 +41,7 @@ use strict;
 ##==============================================================================
 ## Globals & Constants
 
-our $VERSION = "0.11.001";
+our $VERSION = "0.11.002";
 our @ISA = qw(DiaColloDB::Client);
 
 ## $PGOOD_DEFAULT
@@ -958,11 +958,11 @@ sub create {
   }
 
   CORE::close($atokfh)
-      or $coldb->logconfess("create(): failed to temporary attribute-token-file $atokfile: $!");
+      or $coldb->logconfess("create(): failed to close temporary attribute-token-file $atokfile: $!");
   CORE::close($tokfh)
-      or $coldb->logconfess("create(): failed to temporary token-file $tokfile: $!");
+      or $coldb->logconfess("create(): failed to close temporary token-file $tokfile: $!");
   CORE::close($vtokfh)
-      or $coldb->logconfess("create(): failed to temporary tdf-token-file $vtokfile: $!");
+      or $coldb->logconfess("create(): failed to close temporary tdf-token-file $vtokfile: $!");
   my $ntok_out = $toki_out;
   my $ptokbad = $ntok_in ? sprintf("%.2f%%",100*($ntok_in-$ntok_out)/$ntok_in) : 'nan%';
   $coldb->vlog($coldb->{logCreate}, "create(): assigned $nt term tuple-IDs to $ntok_out of $ntok_in tokens (pruned $ptokbad)");
@@ -1749,12 +1749,12 @@ sub parseQuery {
   ##-- compat: parse into attribute-local requests $areqs=[[$attr1,$areq1],...]
   my $sepre  = qr{[\s\,]};
   my $charre = qr{(?:\\[^ux0-9]|[\w\x{80}-\x{ffff}])};
-  my $attrre = qr{(?:\$?${charre}+)};
+  my $attrre = qr{(?:\$?(?:doc\.)?${charre}+)};
   my $orre   = qr{(?:\s*\|?\s*)};
   my $setre  = qr{(?:(?:${charre}+)(?:${orre}${charre}+)*)};	##-- value: |-separated barewords
   my $regre  = qr{(?:/(?:\\/|[^/]*)/(?:[gimsadlux]*))};		##-- value regexes
   my $valre  = qr{(?:${setre}|${regre})};
-  my $reqre  = qr{(?:(?:${attrre}[:=])?${valre})};
+  my $reqre  = qr{(?:(?:${attrre}(?:[:=]${valre})?)|${valre})};
   if (!$areqs
       && ($ddcmode <= 0)			##-- allow native parsing?
       && $req =~ m/^${sepre}*			##-- initial separators (optional)
