@@ -36,9 +36,10 @@ our %EXPORT_TAGS =
      env   => [qw(env_set env_push env_pop)],
      pack  => [qw(packsize packsingle packFilterFetch packFilterStore)],
      math  => [qw($LOG2 log2 min2 max2 lmax lmin lsum)],
-     list  => [qw(luniq sluniq xluniq)],
+     list  => [qw(luniq sluniq xluniq lcounts)],
      regex => [qw(regex)],
      html  => [qw(htmlesc)],
+     ddc   => [qw(ddc_escape)],
      time  => [qw(s2hms s2timestr timestamp)],
      file  => [qw(file_mtime file_timestamp du_file du_glob copyto moveto copyto_a cp_a fh_flush fh_reopen)],
      si    => [qw(si_str)],
@@ -468,6 +469,14 @@ sub xluniq {
 	 ];
 }
 
+## \%l_counts = lcounts(\@l)
+##  + return occurrence counts for elements of @l
+sub lcounts {
+  my %counts = qw();
+  ++$counts{$_} foreach (grep {defined($_)} @{$_[0]//[]});
+  return \%counts;
+}
+
 ##==============================================================================
 ## Functions: regexes
 
@@ -505,6 +514,20 @@ sub htmlesc {
   return $str;
 }
 
+##==============================================================================
+## Functions: ddc
+
+## $escaped_str = ddc_escape($str)
+## $escaped_str = ddc_escape($str, $addQuotes=1)
+sub ddc_escape {
+  shift(@_) if (UNIVERSAL::isa($_[0],__PACKAGE__));
+  return $_[0] if ($_[0] =~ /^[a-zA-Z][a-zA-Z0-9]*$/s); ##-- bareword ok
+  my $s = shift;
+  $s =~ s/\\/\\\\/g;
+  $s =~ s/\'/\\'/g;
+  return !exists($_[1]) || $_[1] ? "'$s'" : $s;
+}
+
 ##==============================================================================
 ## Functions: time
 
